/*# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image
#                        pour la Sant�)
# Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
# Previous Authors : Laurent Guigues, Jean-Pierre Roux
# CreaTools website : www.creatis.insa-lyon.fr/site/fr/creatools_accueil
#
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENSE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------ */

#include "bbmaracasvisuPlotterView.h"
#include "bbcreaMaracasVisuPackage.h"

#include "pPlotterScaleX.h"
#include "pPlotterScaleY.h"


namespace bbcreaMaracasVisu
{

BBTK_ADD_BLACK_BOX_TO_PACKAGE(creaMaracasVisu,PlotterView)
BBTK_BLACK_BOX_IMPLEMENTATION(PlotterView,bbtk::WxBlackBox);

//---------------------------------------------------
void PlotterView::Process()
{
//	if (firsttime==true)
//	{
//		firsttime=false;
		
	
	printf("EED PlotterView::Process %d %d \n ", bbGetInputInY().size(),bbGetInputInY2().size());
	
	pGraphicalFunction *pGF = mwxwidget->getFunction(0);
	pGraphicalFunction *pGF2 = mwxwidget->getFunction(1);

	   if ( (bbGetInputInY().size()!=0)  &&  (bbGetInputInY2().size()==0) )
	   {
		   bbSetInputInY2( bbGetInputInY() );
	   } 
	
		if ( (bbGetInputInY().size()!=0) && (pGF==NULL) )		
		{
			// avoid 'taking address of temporary '
			//pGraphicalFunction *pGF = mwxwidget->getFunctionForVectors( &(bbGetInputInX()), &(bbGetInputInY()) ); // taking address of temporary ! JPRx
		
			std::vector<double> inX;
			if (bbGetInputInX().size()==0)
			{
				int i,size=bbGetInputInY().size();
				for(i=0;i<size;i++)
				{
					inX.push_back((double)i);
				} // for
			} else {
				inX = bbGetInputInX();
			} // if bbGetInputInX
			std::vector<double> inY = bbGetInputInY();
		 
	/*
			std::vector<double> inX;
			std::vector<double> inY;
			inX.push_back(0); 			inX.push_back(200);
			inY.push_back(0); 			inY.push_back(1);
	 */
			pGF = mwxwidget->getFunctionForVectors( &inX, &inY ); 
			mwxwidget->addFunction(pGF);		
		} // if bbGetInputInY

		if ((bbGetInputInY2().size()!=0)&& (pGF2==NULL)		)
		{
			// avoid 'taking address of temporary '
			//pGraphicalFunction *pGF = mwxwidget->getFunctionForVectors( &(bbGetInputInX()), &(bbGetInputInY()) ); // taking address of temporary ! JPRx
		
			std::vector<double> inX2;
			if (bbGetInputInX().size()==0)
			{
				int i,size=bbGetInputInY2().size();
				for(i=0;i<size;i++)
				{
					inX2.push_back((double)i);
				} // for
			} else {
				inX2 = bbGetInputInX();
			} // if bbGetInputInX
			std::vector<double> inY2 = bbGetInputInY();
		/*
			std::vector<double> inX2;
			std::vector<double> inY2;
			inX2.push_back(0); 			inX2.push_back(200);
			inY2.push_back(0); 			inY2.push_back(1);
		 */
			pGF2 = mwxwidget->getFunctionForVectors( &inX2, &inY2 ); 
			mwxwidget->addFunction(pGF2);		
		} // if bbGetInputInY
		
//	} // if firsttime
	
	if (pGF!=NULL)	
	{
		// -Erasing  points-
		int i,sizePoints		= pGF->getSizePoints();
		for (i=1; i<sizePoints;i++)
		{
			pGF->deletePointAt(1);  //  EED here NOT 0 is 1
		} // for i
		// -Add Points-
		double px;
		sizePoints=bbGetInputInY().size();
		for (i=0; i<sizePoints;i++)
		{
			if (i<bbGetInputInX().size())
			{
				px = bbGetInputInX()[i]; 
			} else {
				px = i;
			}	
			pGF->AddNewPoint( px, bbGetInputInY()[i] );
		}		
	}// if pGF


	if (pGF2!=NULL)	
	{
		// -Erasing  points-
		int i,sizePoints		= pGF2->getSizePoints();
		for (i=0; i<sizePoints;i++)
		{
			pGF2->deletePointAt(1);   //  EED here NOT 0 is 1
		} // for i
		// -Add Points-
		double px;
		sizePoints=bbGetInputInY2().size();
		for (i=0; i<sizePoints;i++)
		{
			if (i<bbGetInputInX().size())
			{
				px = bbGetInputInX()[i]; 
			} else {
				px = i;
			}	
			pGF2->AddNewPoint( px, bbGetInputInY2()[i] );
		}		
	}// if pGF
		
	mwxwidget->UpdateAll();
	
	
}

	
//---------------------------------------------------
void PlotterView::CreateWidget(wxWindow* parent)
{
   bbtkDebugMessageInc("Core",9,"PlotterView::CreateWxWindow()"<<std::endl);
    
   mwxwidget = new pPlotterWindow( parent , -1,wxDefaultPosition,  wxDefaultSize,  0 );
   mwxwidget->AddLayer(new pPlotterScaleX());
   mwxwidget->AddLayer(new pPlotterScaleY());

	mwxwidget->setMinScrY(0);
	mwxwidget->setMaxScrY(5000);

	
	   bbtkDebugDecTab("Core",9);
   bbSetOutputWidget( mwxwidget );
}

//---------------------------------------------------
void PlotterView::bbUserSetDefaultValues()
{
	mwxwidget = NULL;
	firsttime=true;
}
	
	//-----------------------------------------------------------------	
	void PlotterView::bbUserInitializeProcessing()
	{
	}
	
	//-----------------------------------------------------------------	
	void PlotterView::bbUserFinalizeProcessing()
	{
	}
	
	//-----------------------------------------------------------------	

}
// EO namespace bbcreaMaracasVisu


